<?php
/**
 * Utility
 * @package     Me
 * @copyright   Copyright (C) 2011 MicroEngine Inc.
 */
class Me_Util
{
    /** form id を格納するセッション名 */
    const SESSION_FORM_KEY = 'ME_FORMS';

    /**
     * IE判定
     * @var boolean
     */
    private static $is_ie_flag = null;
    /**
     * double_encodeオプションの有無
     * @var boolean
     */
    public static $exist_double_encode = true;

    /**
     * 初期化
     */
    static public function init()
    {
        if (PHP_VERSION_ID < 50203) {
            self::$exist_double_encode = false;
        }
    }

    /**
     * html特殊文字をエンティティ化する
     * @param mixed(string|array) $str エンティティ化対象文字列または配列
     * @return mixed(string|array) 特殊文字をエンティティ化した、文字列または配列
     */
    static public function html_escape($str)
    {
        if (is_array($str)) {
            return array_map(array('Me_Util', 'html_escape'), $str);
        } else if (is_object($str) || is_null($str)) {
            return '';
        }
        if (self::$exist_double_encode) {
            // double_encodeオプションが使える場合
            return htmlspecialchars($str, ENT_QUOTES, 'UTF-8', false);
        } else {
            // double_encodeオプションが使えない場合は、一旦デコードしてからエンコードする
            return htmlspecialchars(html_entity_decode($str, ENT_QUOTES, 'UTF-8'), ENT_QUOTES, 'UTF-8');
        }
    }

    /**
     * htmlエンティティをデコードする
     * @param mixed(string|array) $str デコード対象文字列または配列
     * @return mixed(string|array) デコードした、文字列または配列
     */
    static public function html_decode($str)
    {
        if (is_array($str)) {
            return array_map(array('Me_Util', 'html_decode'), $str);
        } else if (is_object($str) || is_null($str)) {
            return '';
        }
        return html_entity_decode($str, ENT_QUOTES, 'UTF-8');
    }

    /**
     * html escapeしたうえで、改行をbrタグに変換する
     * @param string $str 出力する文字列
     * @param boolean $is_xhtml
     * @return string
     */
    static public function nl2br_escape($str, $is_xhtml = false)
    {
        if (PHP_VERSION_ID >= 50300) {
            return nl2br(Me_Util::html_escape($str), $is_xhtml);
        } else {
            if ($is_xhtml) {
                return nl2br(Me_Util::html_escape($str));
            } else {
                // nl2brが書きだしたBRタグ（'<br />'）にのみ対応する
                return str_replace('<br />', '<br>', nl2br(Me_Util::html_escape($str)));
            }
        }
    }

    /**
     * 文字列の見た目の長さで丸める
     * 半角は1文字、全角は2文字と計算して丸める。
     * @param string $str 対象の文字列
     * @param integer $width 丸める文字数
     * @return string
     */
    static public function trim_to($str, $width)
    {
        $trim_marker = Me_Config::get_page('trim_marker');
        return mb_strimwidth($str, 0, $width, $trim_marker, Template_Logic::CHAR_CODE);
    }

    /**
     * Token生成
     * @return string $_SESSION['_token'] トークン文字列
     */
    static public function get_token()
    {
        if (!isset($_SESSION['_token'])) {
            $_SESSION['_token'] = sha1(session_id() . microtime());
        }
        return $_SESSION['_token'];
    }

    /**
     * Token確認
     */
    static public function check_token()
    {
        // postの場合はチェック
        if ($_SERVER['REQUEST_METHOD'] == 'POST') {
            // tokenが一致しなければ不正リクエストとする。
            if (!isset($_SESSION['_token']) || $_SESSION['_token'] !== $_POST['_token']) {
                throw new Me_Exception(400);
            }
        }
    }

    /**
     * form id生成
     * @return integer $_form_id
     */
    static public function generate_form_id()
    {
        if (!isset($_SESSION[self::SESSION_FORM_KEY])) {
            $_SESSION[self::SESSION_FORM_KEY] = array();
        }
        do {
            $_form_id = mt_rand();
        } while (isset($_SESSION[self::SESSION_FORM_KEY][$_form_id]));
        $_SESSION[self::SESSION_FORM_KEY][$_form_id] = array();
        return $_form_id;
    }

    /**
     * form id取得
     * @return integer $_form_id
     */
    static public function get_form_id()
    {
        if ($_SERVER['REQUEST_METHOD'] === 'POST') {
            $_form_id = isset($_POST['_form_id']) ? $_POST['_form_id'] : null;
        } else {
            $_form_id = isset($_GET['_form_id']) ? $_GET['_form_id'] : null;
        }
        return $_form_id;
    }

    /**
     * form idで管理しているformを取得
     * @param string $_form_id
     * @return object
     */
    static public function get_form($_form_id = null)
    {
        if ($_form_id === null) {
            $_form_id = Me_Util::get_form_id();
        }
        if (isset($_SESSION[self::SESSION_FORM_KEY][$_form_id])) {
            return $_SESSION[self::SESSION_FORM_KEY][$_form_id];
        } else {
            throw new Me_Exception(500);
        }
    }

    /**
     * formオブジェクトをセッションに保存
     * @param object $form
     * @param integer $_form_id
     */
    static public function set_form($form, $_form_id = null)
    {
        if ($_form_id === null) {
            $_form_id = Me_Util::get_form_id();
        }
        $_SESSION[self::SESSION_FORM_KEY][$_form_id] = $form;
    }

    /**
     * formをセッションから削除
     * @param integer $_form_id
     */
    static public function unset_form($_form_id = null)
    {
        if ($_form_id === null) {
            $_form_id = Me_Util::get_form_id();
        }
        unset($_SESSION[self::SESSION_FORM_KEY][$_form_id]);
    }

    /**
     * クエリーストリング追加
     * ?の有無を確認して、?か&で$query_stringを連結する。
     * @param string $url
     * @param string $query_string
     * @return string $url
     */
    static public function add_query_string($url, $query_string)
    {
        if (strpos($url, '?') === false) {
            $url .= '?' . $query_string;
        } else {
            $url .= '&' . $query_string;
        }
        return $url;
    }

    /**
     * POSTされた値をオブジェクトにセットする。
     * @param object $object
     * @param array $mapping 'オプジェクトのプロパティ' => 'Formのkey'
     */
    static public function post_data_to_object($object, $mapping)
    {
        if ($_SERVER['REQUEST_METHOD'] !== 'POST') {
            // リクエストメソッドがPOSTでなければエラーとする。
            throw new Me_Exception(401);
        }
        foreach ($mapping as $property => $_form_key) {
            $object->$property = $_POST[$_form_key];
        }
    }

    /**
     * postされた値を取得
     * @param string $key
     * @param string $default
     * @return string
     */
    static public function get_post_data($key, $default = null)
    {
        return isset($_POST[$key]) ? $_POST[$key] : $default;
    }

    /**
     * getで渡されたquery stringを取得
     * @param string $key
     * @param string $default
     * @return string
     */
    static public function get_query_string($key, $default = null)
    {
        return (strlen($_GET[$key]) > 0) ? $_GET[$key] : $default;
    }

    /**
     * magic_quotes_gpc対策
     */
    static public function against_magic_quotes()
    {
        if (get_magic_quotes_gpc()) {
            $_GET = self::stripslashes_deep($_GET);
            $_POST = self::stripslashes_deep($_POST);
            $_COOKIE = self::stripslashes_deep($_COOKIE);
            $_REQUEST = self::stripslashes_deep($_REQUEST);
        }
    }

    /**
     * 再帰的にstripslashesを実行する
     * @var array $arr
     * @return array $arr
     */
    static public function stripslashes_deep($arr)
    {
        return is_array($arr) ?
            array_map(array('Me_Util', 'stripslashes_deep'), $arr) :
            stripslashes($arr);
    }

    /**
     * リダイレクト処理
     * @param string $path
     */
    static public function redirect($path)
    {
        header('Location: ' . $path);
        exit;
    }

    /**
     * リフレッシュ処理
     */
    static public function refresh()
    {
        header('Location: ' . $_SERVER['REQUEST_URI']);
        exit;
    }

    /**
     * ファイル内容取得
     * @param string $file_path ファイル名のフルパス
     * @return string $ret ファイルの内容
     */
    static public function get_file($file_path)
    {
        // ファイルの存在確認
        if (!file_exists($file_path)) {
            throw new Me_Exception(302, $file_path);
        }
        // ディレクトリの場合はエラーとする。
        if (is_dir($file_path)) {
            throw new Me_Exception(303, $file_path);
        }
        $ret = file_get_contents($file_path);
        if ($ret === false) {
            throw new Me_Exception(303, $file_path);
        }
        return $ret;
    }

    /**
     * ファイル書き出し
     * @param string $file_path サーバ内のフルパス
     * @param string $file 出力するファイルの内容
     * @param boolean $mkdir ディレクトリがなければ作成する
     * @return boolean
     */
    static public function write_file($file_path, $file, $mkdir = true)
    {
        // Windowsの場合は文字コード変換
        if (PHP_OS === 'WINNT' || PHP_OS === 'WIN32') {
            $file_path = mb_convert_encoding($file_path, 'SJIS-win', 'UTF-8');
        }
        //出力ディレクトリの存在確認
        $dir_path = dirname($file_path);
        if (!file_exists($dir_path)) {
            if ($mkdir) {
                // ディレクトリを再帰的に作成する
                Me_Util::mkdir($dir_path, 0755, true);
            } else {
                throw new Me_Exception(300, $dir_path);
            }
        }
        $exist = file_exists($file_path);
        if ($exist) {
            // ファイルがあれば、そのファイルの書き込み権限を確認
            if (!is_writable($file_path)) {
                throw new Me_Exception(305, $file_path);
            }
        } else {
            // ファイルがなければ、ディレクトリの書き込み権限を確認
            if (!is_writable(dirname($file_path))) {
                throw new Me_Exception(305, dirname($file_path));
            }
        }
        if (file_put_contents($file_path, $file, LOCK_EX) === false) {
            throw new Me_Exception(304, $file_path);
        }
        if (!$exist) {
            // ファイルパーミッション・グループ変更
            Me_Util::change_file_permission($file_path);
        }
        return true;
    }

    /**
     * 相対パス取得
     * ２つのパスから相対パスを取得する。
     * @param string $from_path 基点となるパス
     * @param string $dest_path 目的地となるパス
     * @return string $ret
     */
    static public function get_relative_path($from_path, $dest_path)
    {
        $ret = '';
        // ディレクトリの場合は、末尾に / をつける。
        if (is_dir($from_path)) {
            $from_path = rtrim($from_path, '/') . '/';
        }
        if (is_dir($dest_path)) {
            $dest_path = rtrim($dest_path, '/') . '/';
        }

        // パスを配列に変換
        $from_arr = explode('/', $from_path);
        $dest_arr = explode('/', $dest_path);
        // 重複するパスを削除する
        while (true) {
            // 異なるファイル・ディレクトリが現れたらループ終了
            if (!isset($from_arr[0]) || !isset($dest_arr[0]) || $from_arr[0] !== $dest_arr[0]) {
                break;
            }
            // 配列の先頭要素を削除
            array_shift($from_arr);
            array_shift($dest_arr);
        }
        // 基点配列の最後の要素はファイル（相当）なのでその分を引いた数を取得する
        $size = count($from_arr) - 1;
        if ($size > 0) {
            // その数ぶん'../'を追加する
            $ret .= str_repeat('../', $size);
        }
        // 残りの目的地配列を'/'で連結する
        $ret .= implode('/', $dest_arr);
        return $ret;
    }

    /**
     * IE判定
     * @return boolean
     */
    public static function is_ie()
    {
        if (self::$is_ie_flag === null) {
            self::$is_ie_flag = (strpos($_SERVER["HTTP_USER_AGENT"], 'MSIE') === false) ? false : true;
        }
        return self::$is_ie_flag;
    }

    /**
     * 変数表現をファイルに書きだす
     * @param mixed $var
     * @param string $file_path
     */
    public static function write_var_file($var, $file_path)
    {
        $logic_file = '<?php $var = ' . var_export($var, true) . ';';
        // ファイル書き出し
        Me_Util::write_file($file_path, $logic_file);
    }

    /**
     * 変数表現のファイルを読み込んでその変数を返す
     * @param string $file_path
     * @return mixed $var
     */
    public static function read_var_file($file_path)
    {
        $var = null;
        include $file_path;
        return $var;
    }

    /**
     * 古い一時ファイル削除
     */
    public static function unlink_old_tmpfile()
    {
        // 一時ディレクトリ
        $dir = UPLOAD_ROOT . Page_Form::TEMPORARY_DIRECTORY . '/';

        // ディレクトリの存在確認
        if (!file_exists($dir)) {
            return;
        }

        // ディレクトリハンドルのオープン
        if ($handle = opendir($dir)) {
            while (false !== ($file = readdir($handle))) {
                // 86400秒（1日）以前の通常ファイルは削除
                if (filemtime($dir . $file) < (time() - 86400)
                    && is_file($dir . $file)
                    && $file !== 'index.html'
                    && substr($file, 0, 1) !== '.') {
                    unlink($dir . $file);
                }
            }
            closedir($handle);
        }
    }

    /**
     * 複数行文字列を各行に分割して配列にして返す。
     * 前後の空文字は削除。空白行は削除。
     * @param string $text
     * @return array $lines 
     */
    public static function get_lines($text)
    {
        $lines = array();
        // 各行に分割
        $text_lines = preg_split('#(\r\n|\r|\n)#', $text);

        foreach ($text_lines as $line) {
            // 前後空白を取り除く
            $line = trim($line);
            // 文字がある場合だけ結果リストに追加
            if (strlen(trim($line)) > 0) {
                $lines[] = $line;
            }
        }
        return $lines;
    }

    /**
     * カンマ区切りの文字列を分割して配列で返す
     * @param string $str
     * @param boolean $flip
     * @return array $value_list
     */
    public static function get_array($str, $flip = false)
    {
        if (strlen($str) < 1) {
            return array();
        }
        $value_list = explode(',', $str);
        foreach ($value_list as $key => &$value) {
            if (strlen($value) > 0) {
                //　前後の空白は削除
                $value = trim($value);
            } else {
                // 空文字は削除
                unset($value_list[$key]);
            }
        }
        if ($flip) {
            $value_list = array_flip($value_list);
        }
        return $value_list;
    }

    /**
     * pathinfoのラッパー
     * 5.2未満で未対応の"filename"も含める
     * @param string $path
     * @return array $path_parts
     */
    public static function pathinfo($path)
    {
        $path_parts = pathinfo($path);
        if (PHP_VERSION_ID < 50200) {
            $path_parts['filename'] = substr($path_parts['basename'], 0, strrpos($path_parts['basename'], '.'));
        }
        return $path_parts;
    }

    /**
     * アップロードファイル保存
     * POSTしたファイルを、指定した名前で、指定した場所に保存する。
     * @param string $source_file
     * @param string $save_file_path
     */
    public static function save_file($source_file, $save_file_path)
    {
        // 保存先のディレクトリがなければ作成
        if (!is_dir(dirname($save_file_path))) {
            $dir_path = dirname($save_file_path);
            Me_Util::mkdir($dir_path);
        }

        // ファイルを移動する
        if (!move_uploaded_file($source_file, $save_file_path)) {
            throw new Me_Exception(604);
        }
        // ファイルパーミッション・グループ変更
        Me_Util::change_file_permission($save_file_path);
    }

    /**
     * ファイルリネーム処理
     */
    public static function rename_file($old_file_path, $new_file_path)
    {
        if (!rename($old_file_path, $new_file_path)) {
            throw new Me_Exception(307, array($old_file_path, $new_file_path));
        }
    }
    
    /**
     * ファイルパーミッション・グループを変更する
     * config.iniに設定がある場合のみ実行される
     * @param string $file_path
     */
    public static function change_file_permission($file_path)
    {
        // ファイルパーミッション設定があれば変更
        $file_permission = Me_Config::get('file_permission', 'file_system');
        if (strlen($file_permission) > 0) {
            if (!chmod($file_path, octdec($file_permission))) {
                throw new Me_Exception(308, $file_path);
            }
        }
        // ファイルグループ設定があれば変更
        $file_group = Me_Config::get('file_group', 'file_system');
        if (strlen($file_group) > 0) {
            if (!chgrp($file_path, $file_group)) {
                throw new Me_Exception(310, $file_path);
            }
        }
    }

    /**
     * ディレクトリパーミッション・グループを変更する
     * config.iniに設定がある場合のみ実行される
     * @param string $dir_path
     */
    public static function change_dir_permission($dir_path)
    {
        // ディレクトリパーミッション設定があれば変更
        $directory_permission = Me_Config::get('directory_permission', 'file_system');  
        if (strlen($directory_permission) > 0) {
            if (!chmod($dir_path, octdec($directory_permission))) {
                throw new Me_Exception(309, $dir_path);
            }
        }
        // ディレクトリグループ設定があれば変更
        $directory_group = Me_Config::get('directory_group', 'file_system');
        if (strlen($directory_group) > 0) {
            if (!chgrp($dir_path, $directory_group)) {
                throw new Me_Exception(311, $dir_path);
            }
        }
    }
    
    /**
     * mkdir wrapper
     * 再帰的にディレクトリパーミッション・グループを設定する。
     * @param string $pathname
     * @param int $mode
     * @param boolean $recursive
     * @throws Me_Exception
     */
    public static function mkdir($pathname, $mode = 0775, $recursive = false)
    {
        $before_dir = $pathname;
        // 再帰処理の場合
        if ($recursive) {
            // 処理前の既存ディレクトリを確認
            while (!file_exists($before_dir) && strlen($before_dir) > 1) {
                $before_dir = dirname($before_dir);
            }
        }
        
        if (!mkdir($pathname, $mode, $recursive)) {
            throw new Me_Exception(301, $pathname);
        }
        if ($recursive) {
            // 再帰処理の場合
            while ($pathname != $before_dir && strlen($pathname) > 1) {
                // ディレクトリパーミッション・グループ変更
                Me_Util::change_dir_permission($pathname);
                $pathname = dirname($pathname);
            }
        } else {
            // ディレクトリパーミッション・グループ変更
            Me_Util::change_dir_permission($pathname);
        }
        return true;
    }
}