<?php
/**
 * Page Logic
 * @package     Page
 * @copyright   Copyright (C) 2011 MicroEngine Inc.
 */
class Page_Logic
{
    /** Page Type of Directory */
    const PAGE_TYPE_DIRECTORY = '1';
    /** Page Type of Template */
    const PAGE_TYPE_TEMPLATE = '2';
    /** Page Type of File */
    const PAGE_TYPE_FILE = '3';
    /** Page Type of Other File */
    const PAGE_TYPE_OTHER = '4';
    /** 本稿登録 */
    const SAVE_PAGE = 'page';
    /** 下書き保存 */
    const SAVE_DRAFT = 'page_draft';

    /**
     * DAOオブジェクト
     * @var Page_Dao
     */
    private $dao;

    /**
     * コンストラクタ
     */
    public function __construct()
    {
        $this->dao = new Page_Dao();
    }

    /**
     * SELECT
     * 1レコードのオブジェクトを返す
     * @param string $_id _idフィールド値
     * @param string $class_name クラス名
     * @return object $obj
     */
    public function select($_id, $class_name = null)
    {
        return $this->dao->select($_id, $class_name);
    }

    /**
     * ページリスト取得
     * @param string $dir
     * @return array $rows
     */
    public function select_page_list($dir = '')
    {
        return $this->dao->select_page_list($dir);
    }

    /**
     * ページ情報取得
     * @param string $page_path
     * @return array $row
     */
    public function select_page_info($page_path = '')
    {
        return $this->dao->select_page_info($page_path);
    }

    /**
     * 同期対象リスト取得
     * @return array
     */
    public function get_sync_list()
    {
        $delete_list = array();
        // ファイルシステム上から対象となるページ一覧を取得
        $file_list = array();
        $this->get_page_list($file_list);

        // DB登録済みのページリストを取得
        $db_list = $this->dao->select_registered_list();

        // DB登録済みを除外
        foreach ($db_list as $db_entry) {
            // ファイルシステム上になくて、テンプレート化していなファイルファイルは削除する。
            if (!isset($file_list[$db_entry['page_path']])
                && ($db_entry['page_type'] === self::PAGE_TYPE_FILE || $db_entry['page_type'] === self::PAGE_TYPE_DIRECTORY )) {
                $page = new Page();
                $page->_id = $db_entry['_id'];
                $page->page_path = $db_entry['page_path'];
                $delete_list[] = $page;
            }
            // チェック済みはunsetする。unsetされずに残ったファイルは追加する。
            unset($file_list[$db_entry['page_path']]);
        }

        return array($file_list, $delete_list);
    }

    /**
     * ページリスト取得
     * 指定ディレクトリ配下のページの一覧を再帰的に取得する。
     * @param array &$page_list
     * @param string $directory SITE_ROOTからの相対パス
     * @param integer $depth ディレクトリ深さ
     * @return array $List
     */
    private function get_page_list(&$page_list, $directory = '/', $depth = 0)
    {
        // 現在ディレクトリのファイルシステム上の絶対パス
        $current_dir_path = SITE_ROOT . $directory;

        // 現在ディレクトリ内のファイルとディレクトリのリスト
        $current_file_list = scandir($current_dir_path);

        // 管理対象外ディレクトリを取得
        $exclude_dir = array_flip(Me_Config::get_array('exclude_dir', 'page'));

        // CMS管理対象拡張子
        $extension_list = Me_Config::get_array('extensions', 'page');
        // ファイルマネージャー管理対象拡張子
        $file_manager_extension_list = Me_Config::get_array('extensions', 'file');

        foreach ($current_file_list as $file_name) {
            // .で始まるファイルは除外。.htaccessは例外。
            if (substr($file_name, 0, 1) === '.' && $file_name !== '.htaccess') {
                continue;
            }
            // Windowsの場合は文字コード変換
            if (PHP_OS === 'WINNT' || PHP_OS === 'WIN32') {
                $file_name = mb_convert_encoding($file_name, 'UTF-8', 'SJIS-win');
            }

            // ファイルシステム上の絶対パス
            $file_path = $current_dir_path . $file_name;

            // SITE_ROOTからのパス
            $page_path = $directory . $file_name;

            // meディレクトリまたは、アップロードディレクトリは除外
            if ($file_path === ME_ROOT || $file_path === UPLOAD_ROOT || isset($exclude_dir[$page_path])) {
                continue;
            }

            if (is_dir($file_path)) {
                // ディレクトリは "/" で終える
                $page_path .= '/';
                // pageオブジェクト作成
                $page = new Page();
                $page->directory = $directory;
                $page->extension = '';
                $page->file_name = $file_name;
                $page->page_path = $page_path;
                $page->page_type = self::PAGE_TYPE_DIRECTORY;

                // ファイルリストに追加
                $page_list[$page_path] = $page;

                // 再帰処理
                $this->get_page_list($page_list, $page_path, ++$depth);
            } else {
                $path_parts = pathinfo($file_name);

                if (!empty($extension_list)
                    && preg_match('/^(' . implode('|', $extension_list) . ')$/i', $path_parts['extension'])) {
                    $file_name = substr($file_name, 0, strlen($file_name) - strlen($path_parts['extension']) - 1);

                    // pageオブジェクト作成
                    $page = new Page();
                    $page->directory = $directory;
                    $page->extension = '.' . $path_parts['extension'];
                    $page->file_name = $file_name;
                    $page->page_path = $page_path;
                    $page->page_type = self::PAGE_TYPE_FILE;

                    // ファイルリストに追加
                    $page_list[$page_path] = $page;
                } else if (!empty($file_manager_extension_list)
                    && preg_match('/^(' . implode('|', $file_manager_extension_list) . ')$/i', $path_parts['extension'])) {
                    $file_name = substr($file_name, 0, strlen($file_name) - strlen($path_parts['extension']) - 1);

                    // pageオブジェクト作成
                    $page = new Page();
                    $page->directory = $directory;
                    $page->extension = '.' . $path_parts['extension'];
                    $page->file_name = $file_name;
                    $page->page_path = $page_path;
                    $page->page_type = self::PAGE_TYPE_OTHER;

                    // ファイルリストに追加
                    $page_list[$page_path] = $page;
                }
            }
        }
    }

    /**
     * ページリスト登録
     * @param array $page_list
     */
    public function insert_page_list($page_list)
    {
        // DBに登録
        foreach ($page_list as $page) {
            $this->dao->insert($page);
        }
    }

    /**
     * ページリスト削除
     * @param array $page_list
     */
    public function delete_page_list($page_list)
    {
        // DBから削除
        foreach ($page_list as $page) {
            $this->dao->delete($page);
        }
    }

    /**
     * ページ編集
     * @param string $page_id
     * @param string $template_id
     * @param stirng $draft_id
     * @param stirng $draft_edit
     * @param string $back
     * @return Page_Form $form
     */
    public function edit_page($page_id, $template_id = null, $draft_id = null, $draft_edit = null, $back = null)
    {
        $template_logic = new Template_Logic();
        // ページ情報取得
        if (empty($page_id)) {
            $form = new Page_Form();
        } else {
            $form = $this->dao->select($page_id, 'Page_Form');
        }
        $form->back = $back;

        // 下書き編集モード
        $form->draft_edit = ($draft_edit);
        $form->draft_id = $draft_id;
        // 下書き情報取得
        if ($form->draft_edit) {
            $page_draft_dao = new Page_Draft_Dao();
            $page_draft = $page_draft_dao->select($draft_id);
            $form->file_name = $page_draft->file_name;
            $form->title = $page_draft->title;
            $form->keywords = $page_draft->keywords;
            $form->description = $page_draft->description;
            $form->note = $page_draft->note;
            $form->comment = $page_draft->comment;
        }

        if (!empty($template_id)) {
            /* @var $template Template */
            $template = $template_logic->select($template_id);
            // シングルタイプテンプレートでテンプレートIDが指定されているばあいはエラーにする
            if ($template->template_type === Template_Logic::TYPE_SINGLE) {
                throw new Me_Exception(606);
            }
            $form->template_id = $template->_id;
        }

        // テンプレート情報取得
        $template_form = $template_logic->get_template_form_by_page($form);

        if (empty($page_id)) {
            $this->set_file_info($template_form, $form);
        }

        // ページ情報からテンプレートを作成
        $template_logic->get_template_file($template_form);
        $this->update_header($template_form);
        // parse処理
        $parser = new Template_Parser();
        $form->document = $parser->parse($template_form->source_code);
        $form->template_form = $template_form;

        // 予約IDの値セット
        if ($form->title === null) {
            $title_tag = $form->document->get_element_by_id('_title');
            if ($title_tag !== null && !empty($title_tag->initial_value)) {
                $form->title = Me_util::html_decode($title_tag->initial_value);
            }
        }
        if ($form->keywords === null) {
            $keywords_tag = $form->document->get_element_by_id('_keywords');
            if ($keywords_tag !== null && !empty($keywords_tag->initial_value)) {
                $form->keywords = Me_util::html_decode($keywords_tag->initial_value);
            }
        }
        if ($form->description === null) {
            $description_tag = $form->document->get_element_by_id('_description');
            if ($description_tag !== null && !empty($description_tag->initial_value)) {
                $form->description = Me_util::html_decode($description_tag->initial_value);
            }
        }

        if ($form->draft_edit) {
            // 下書き情報取得
            $metag_dao = new Metag_Draft_Dao();
        } else {
            // ページデータ取得
            $metag_dao = new Metag_Dao();
        }
        $form->data = $metag_dao->get_page_data($page_id);
        return $form;
    }

    /**
     * 入力値チェック
     * @param Page_Form $form
     * @return boolean
     */
    public function validation(Page_Form $form)
    {
        // エラーメッセージを初期化
        $form->is_error = false;
        $form->errors = array();

        // ファイル名チェック
        $this->check_file_name($form);

        // 公開設定
        // 必須チェック
        if (strlen($form->public_flag) < 1) {
            $form->errors['public_flag'] = '公開設定を選択してください';
            $form->is_error = true;
        }
        // 備考チェック
        if (50 < mb_strlen($form->note)) {
            $form->errors['note'] = '50文字以内で入力してください';
            $form->is_error = true;
        }
        // コメントチェック
        if (50 < mb_strlen($form->comment)) {
            $form->errors['comment'] = '50文字以内で入力してください';
            $form->is_error = true;
        }

        // エラー有無確認
        if ($form->is_error) {
            $form->errors['MESSAGE'] = 'エラーが発生しました。';
            return false;
        }
        return true;
    }

    /**
     * ファイル名チェック
     * @param Page_Form $form
     */
    public function check_file_name(Page_Form $form)
    {
        // エラーメッセージを初期化
        $form->is_error = false;
        $form->errors = array();

        // 新ページパスセット
        $form->new_page_path = $form->directory . $form->file_name . $form->extension;

        // 初期設定値の場合はエラーにしない
        if ($form->page_path === $form->new_page_path) {
            return;
        }

        if (strlen($form->file_name) < 1) {
            $form->errors['file_name'] = 'ファイル名を入力して下さい';
            $form->is_error = true;
        } else if (!preg_match('/^[\w-.]+$/', $form->file_name)) {
            // 文字種チェック
            $form->errors['file_name'] = 'ファイル名は、英字、数字、ハイフン( - ),' .
                ' アンダースコア( _ ), ピリオド( . ), のみで入力して下さい';
            $form->is_error = true;
        } else {
            // 重複確認
            $exist_error = false;

            // DB上でファイル確認
            $row = $this->dao->select_by_page_path($form->new_page_path);
            if (!empty($row)) {
                if ($row['page_type'] === self::PAGE_TYPE_FILE) {
                    // 通常ファイルの場合は、上書きを警告
                    $path = Me_Util::html_escape(SITE_PATH . $form->new_page_path);
                    $form->errors['file_name'] = 'ファイルを上書きします。<a href="' . $path . '" target="_blank">【ファイルを新規ウインドウで開く】</a>';
                } else {
                    $exist_error = true;
                }
            } else {
                // DBにない場合はファイルシステムを確認
                $file_path = SITE_ROOT . $form->new_page_path;
                if (file_exists($file_path)) {
                    $exist_error = true;
                }
            }
            if ($exist_error) {
                $form->errors['file_name'] = '指定されたファイルがすでに存在します。';
                $form->is_error = true;
            }
        }
        return;
    }
    
    /**
     * ページの書き込み権限確認
     * @param string $page_path
     * @throws Me_Exception
     */
    public function check_permission($page_path)
    {
        $file_path = SITE_ROOT . $page_path;
        $dir_path = dirname($file_path);
        if (file_exists($file_path)) {
            // ファイルがあれば、ファイルの書き込み権限を確認
            if (!is_writable($file_path)) {
                throw new Me_Exception(305, $file_path);
            }
        } else {
            // ファイルが無ければ、ディレクトリの書き込み権限を確認
            if (!is_writable($dir_path)) {
                throw new Me_Exception(305, $dir_path);
            }
        }
    }

    /**
     * ファイル情報セット
     * @param Template_Form $template_form
     * @param Page_Form $form
     */
    private function set_file_info(Template_Form $template_form, Page_Form $form)
    {
        $path_parts = pathinfo($template_form->template_path);
        $form->page_type = self::PAGE_TYPE_TEMPLATE;
        $dirname = ($path_parts['dirname'] === DIRECTORY_SEPARATOR) ? '/' : $path_parts['dirname'] . '/';
        $form->directory = $dirname;
        $form->extension = '.' . $path_parts['extension'];
        $form->public_flag = '1';

        $file_name = preg_replace(
            '/\.[^\.]*$/', '_', $path_parts['basename']
        );

        // DB上のファイル名取得
        $rows = $this->dao->select_file_name_list($dirname);
        $file_list = array();
        foreach ($rows as $row) {
            $file_list[] = $row['file_name'];
        }
        $file_list = array_flip($file_list);

        $template_form->page_no_seq--;
        do {
            $form->file_name = $file_name . ++$template_form->page_no_seq;
            $file_path = SITE_ROOT . $dirname . $form->file_name . $form->extension;
        } while (file_exists($file_path) || isset($file_list[$form->file_name]));

        $form->page_path = $dirname . $form->file_name . $form->extension;
    }

    /**
     * 編集ビュー出力
     * @param object $node Metag_*
     * @param Page_Form $form
     * @param string $repeat_no
     * @param boolean $set_initial_flag
     */
    public function output_edit_view($node, $form, $repeat_no = null, $set_initial_flag = true)
    {
        // 該当ノードのデータを取り出す
        $row = $form->get_row($node->id, $node->repeat_id, $repeat_no, $node);

        if (!$node->is_block && $node->node_type !== 'html') {
            if ($node->set_initial_value($row, $set_initial_flag, $form->init)) {
                $form->set_row($node->id, $row, $node->repeat_id, $repeat_no);
            }
        }
        $node->edit_view($row);

        // block要素のみ、子を呼び出す
        if ($node->is_block) {
            switch ($node->node_type) {
                case 'repeat':
                    // repeatタグは、持っているdataの分ループさせる。
                    // 空でも１つは書き出す
                    if (empty($row)) {
                        $row['1'] = array();
                        if (!$form->init) {
                            $set_initial_flag = false;
                        }
                    }
                    foreach ($row as $row_repeat_no => $val) {
                        foreach ($node->child_nodes as $child) {
                            $this->output_edit_view($child, $form, $row_repeat_no, $set_initial_flag);
                        }
                    }
                    $set_initial_flag = true;
                    break;
                case 'display':
                    if ($node->edit !== 'off') {
                        foreach ($node->child_nodes as $child) {
                            $this->output_edit_view($child, $form, $repeat_no);
                        }
                    }
                    break;
                default:
                    foreach ($node->child_nodes as $child) {
                        $this->output_edit_view($child, $form, $repeat_no);
                    }
                    break;
            }
        }
    }

    /**
     * head情報更新
     * 文字コード指定やbaseタグを制御
     * 編集用フレームは UTF-8 で出力されるので、HTMLソース上のcharsetもUTF-8に変更する。
     * @param Template_Form $template_form
     */
    public function update_header(Template_Form $template_form)
    {
        // charsetを指定するmetaタグの確認
        // 例：<meta http-equiv="Content-Type" content="text/html; charset=Shift_JIS">
        $template_form->source_code = preg_replace('!<meta.*http-equiv.*charset=.*["\'].*>!i', '<meta http-equiv="Content-Type" content="text/html; charset=UTF-8" />', $template_form->source_code, 1);

        // xmlタグの確認
        /** 例：<?xml version="1.0" encoding="shift_jis"?> */
        $template_form->source_code = preg_replace('!<\?xml.*encoding=["\'].*["\'].*\?>!i', '<?xml version="1.0" encoding="UTF-8"?>', $template_form->source_code, 1);

        // 追加するタグの内容
        $add_tag = '<base href="http://' . $_SERVER['HTTP_HOST'] . SITE_PATH . $template_form->template_path . '">';

        // baseタグやscriptタグを追加する。
        if (preg_match("/<head.*?>/i", $template_form->source_code, $matches)) {
            // <head>が存在すれば、その後にタグを追加する。
            $template_form->source_code = preg_replace(
                '|' . $matches[0] . '|', $matches[0] . $add_tag, $template_form->source_code
            );
        } else if (preg_match("/<html.*?>/i", $html, $matches)) {
            // <html>が存在すれば、その後にタグを追加する。
            $template_form->source_code = preg_replace(
                '/' . $matches[0] . '/', $matches[0] . '<head>' . $add_tag . '</head>', $template_form->source_code
            );
        } else {
            // headタグもhtmlタグも存在しなければ、先頭に追加する。
            $template_form->source_code = "<head>" . $add_tag . "</head>" . $template_form->source_code;
        }

        return;
    }

    /**
     * ページ登録
     * @param Page_Form $form
     * @return boolean
     */
    public function register_page(Page_Form $form)
    {
        if (!$this->validation($form)) {
            return false;
        }
        $draft_dao = new Page_Draft_Dao();

        // 更新で下書きの場合は、pageを更新しない
        $page_updated = (empty($form->_id) || $form->save === self::SAVE_PAGE);

        // 新ページパスの処理
        $old_page_path = $form->page_path;
        if ($page_updated && $form->page_path !== $form->new_page_path) {
            if (file_exists(SITE_ROOT . $old_page_path)) {
                unlink(SITE_ROOT . $old_page_path);
            }
            $new_page = $this->dao->select_by_page_path($form->new_page_path);
            // 新ページがテンプレート化されていないファイルだったらテーブルから削除する。
            if ($new_page !== null && $new_page['page_type'] == self::PAGE_TYPE_FILE) {
                $new_page_obj = new Page;
                $new_page_obj->_id = $new_page['_id'];
                $this->dao->delete($new_page_obj);
            }
            Me_Logger::log(1603, array($form->page_path, $form->new_page_path));
        }
        $form->page_path = $form->new_page_path;
        if (!$page_updated) {
            $form->page_path = $old_page_path;
        }

        // ページ登録
        if (empty($form->_id)) {
            // ページ新規作成（マルチタイプの新規）
            if ($form->save === self::SAVE_DRAFT) {
                // 下書き保存
                $draft_form = clone $form;
                $draft_form->title = '';
                $draft_form->keywords = '';
                $draft_form->description = '';
                $draft_form->note = '';
                $this->dao->insert($draft_form);
                $form->_id = $draft_form->_id;
            } else {
                // 本稿登録
                $this->dao->insert($form);
            }
        } else {
            // ページ更新
            if ($form->save === self::SAVE_PAGE) {
                // 本稿登録
                $this->dao->update($form);
            }
        }
        // 下書き
        if ($form->save === self::SAVE_DRAFT) {
            // 過去の下書きを削除
            $draft_dao->delete_by_page_id($form->_id);
            // 下書きを登録
            $draft = new Page_Draft();
            $draft->set_page($form);
            $draft_dao->insert($draft);
        }
        $page_revision = new Page_Revision();
        if ($form->save === self::SAVE_PAGE) {
            // リビジョン登録
            $page_revision_dao = new Page_Revision_Dao();
            $revision_max_count = (int)Me_Config::get_page('revision_max_count');
            if ($revision_max_count) {
                $page_revision->set_page($form);
                $page_revision_dao->insert($page_revision);
            }
            $page_revision_dao->delete_overflow_data($form->_id, $revision_max_count);
        }

        // 下書き編集で本稿登録した場合は、下書きデータを削除する
        if ($form->draft_edit && $form->save === self::SAVE_PAGE) {
            // 下書きページ削除
            $draft_dao->delete_by_page_id($form->_id);
            // 下書きMEタグ削除
            $metag_draft_dao = new Metag_Draft_Dao();
            $metag_draft_dao->delete_by_page_id($form->_id);
        }

        // 下書きの場合は、ページ新規作成時のみ実行
        // テンプレートパスのページのページパスが変更された場合は、テンプレートにも反映する。
        if ($page_updated && $form->template_form->template_path === $old_page_path && $form->page_path !== $old_page_path) {
            $old_template_path = DATA_ROOT . Template_Logic::TEMPLATE_DIR . $form->template_form->template_path;
            $form->template_form->template_path = $form->page_path;
            $new_template_path = DATA_ROOT . Template_Logic::TEMPLATE_DIR . $form->template_form->template_path;
            Me_Util::rename_file($old_template_path, $new_template_path);
            Me_Logger::log(1613, array($old_page_path, $form->page_path));
        }

        // テンプレート更新
        $template_dao = new Template_Dao();
        $template_dao->update($form->template_form);

        // DB更新
        if ($form->save === self::SAVE_PAGE) {
            $metag_dao = new Metag_Dao();
            $metag_dao->register_array($form, $page_revision->_id);
        } else {
            $metag_draft_dao = new Metag_Draft_Dao();
            $metag_draft_dao->register_array($form);
        }

        // 不要ファイル削除
        $form->exec_unlink_files();

        // 古い一時ファイル削除
        Me_Util::unlink_old_tmpfile();

        if ($form->save === self::SAVE_PAGE) {
            // ページ書き出し
            $page_output = new Page_Output();
            $page_output->output($form->_id);

            // 関連ページ書き出し
            $relation_dao = new Relation_Dao();
            $page_list = $relation_dao->select_relation_page_id($form->_id, $form->template_id);
            foreach ($page_list as $rel_page_row) {
                // $page_outputは再利用する
                $page_output->output($rel_page_row['_id']);
            }
        }
        return true;
    }

    /**
     * ページ削除
     * @param string $page_id
     * @return boolean
     */
    public function delete_page($page_id)
    {
        // 下書き削除
        $this->delete_draft($page_id);
        // 更新履歴削除
        $this->delete_rev($page_id);
        
        $page = $this->dao->select($page_id);

        // 該当ページのmetagデータは削除
        $metag_dao = new Metag_Dao();
        $metag_dao->delete_page($page_id);

        // ページを非公開に設定
        $page->public_flag = 0;
        $this->dao->update($page);

        // 関連ページ書き出し
        $relation_dao = new Relation_Dao();
        $page_list = $relation_dao->select_relation_page_id($page_id, $page->template_id);
        $page_output = new Page_Output();
        foreach ($page_list as $rel_page_row) {
            // $page_outputは再利用する
            $page_output->output($rel_page_row['_id']);
        }

        // テンプレートと同じパスのページでなければ、ページレコードも削除
        $template_dao = new Template_Dao();
        $template = $template_dao->select($page->template_id);
        if ($page->page_path !== $template->template_path) {
            // DBから削除
            $this->dao->delete($page);
        }

        // Relationテーブルから該当テンプレートのレコードを削除
        $relation_dao->delete_from_page($page_id);

        // ファイルを削除する
        $file_path = SITE_ROOT . $page->page_path;
        if (file_exists($file_path)) {
            unlink($file_path);
        }

        return true;
    }

    /**
     * 下書き削除
     * @param string $page_id
     * @return boolean
     */
    public function delete_draft($page_id)
    {
        // 下書きページ削除
        $draft_dao = new Page_Draft_Dao();
        $draft_dao->delete_by_page_id($page_id);

        // 下書きMEタグ削除
        $metag_draft_dao = new Metag_Draft_Dao();
        $metag_draft_dao->delete_by_page_id($page_id);

        return true;
    }

    /**
     * リビジョン削除
     * @param string $page_id
     * @return boolean
     */
    public function delete_rev($page_id)
    {
        // リビジョンページ情報削除
        $revision_dao = new Page_Revision_Dao();
        $revision_dao->delete_by_page_id($page_id);

        // リビジョンMEタグ情報削除
        $metag_revision_dao = new Metag_Revision_Dao();
        $metag_revision_dao->delete_by_page_id($page_id);

        return true;
    }

    /**
     * ページタイトル取得
     * @param boolean $is_edit
     * @param boolean $draft_edit
     * @param string $revision_id
     * @return string
     */
    public function get_page_title($is_edit, $draft_edit = false, $revision_id = null)
    {
        if ($revision_id) {
            $title = 'ページ再編集';
            $class = 'mode_revision';
        } else if ($draft_edit) {
            $title = 'ページ下書き編集';
            $class = 'mode_draft';
        } else {
            if ($is_edit) {
                $title = 'ページ編集';
                $class = 'mode_edit';
            } else {
                $title = 'ページ新規作成';
                $class = 'mode_new';
            }
        }

        return array($title, $class);
    }

    /**
     * リンクページ選択
     * @param string $dir
     */
    public function select_page($dir = '/')
    {
        if ($dir !== null) {
            // ディレクトリで検索
            $rows = $this->dao->select_link_by_dir($dir);
        } else {
            $rows = array();
        }
        return $rows;
    }

    /**
     * 下書き一覧（ユーザー限定）
     * @param integer $user_id
     * @return array $row
     */
    public function get_draft_list($user_id)
    {
        $draft_dao = new Page_Draft_Dao();
        return $draft_dao->select_list($user_id);
    }

    /**
     * 下書き一覧（すべて）
     * @return array $row
     */
    public function get_draft_all_list()
    {
        $draft_dao = new Page_Draft_Dao();
        return $draft_dao->select_all_list();
    }

    /**
     * 更新ページ一覧（ユーザー限定）
     * @param integer $user_id
     * @return array $row
     */
    public function get_updated_page_list($user_id)
    {
        return $this->dao->select_updated_list($user_id);
    }

    /**
     * 更新ページ一覧（すべて）
     * @return array $row
     */
    public function get_updated_page_all_list()
    {
        return $this->dao->select_updated_all_list();
    }

    /**
     * 戻るリンクを返す
     * @param string $back
     * @param string $path
     * @return string $ret
     */
    public function get_back_link($back, $path)
    {
        $ret = '../Main/index?path=' . $path;
        switch ($back) {
            case 'content':
                $ret = '../Content/content_list?path=' . dirname($path);
                break;
            case 'updated':
                $ret = '../Content/updated_page_list';
                break;
            case 'draft':
                $ret = '../Content/draft_list';
                break;
            default:
                break;
        }
        return $ret;
    }
}